#!/usr/bin/perl
use strict;
use warnings;
use Graph::ModularDecomposition;
my $g = new Graph::ModularDecomposition;

my $default_size_node = undef;
my $default_size_edge = undef;
my $input_file = $ARGV[0];
my $output_file = $ARGV[1];

my %cost_edge;
my %cost_node;
my %predecessors;
my %successors;;
my %group;
my %list;
my %list_ini;
my %size;


open( FILE, "< $input_file");

while(<FILE>){
  if(/node\s*\[([^\]]*)\];/){
    $default_size_node = $1;
  }elsif (/edge\s*\[([^\]]*)\];/){
    $default_size_edge = $1;
  }elsif (/^\s*([^ -]*)\s*(\[([^\]]*)\])?;/){
    my $option = $3;
    my $target1 = $1;
    $target1 = $1 if($target1 =~ /^"(.*)"$/);
    $option = 1 unless defined $option;
    $cost_node{"$target1"} = $option;
    $g->add_vertex($target1);
  }

  if (/(\S*)\s*->\s*(\S*)\s*(\[([^\]]*)\])?;/){
    my $option = $4;
    $option = 1 unless defined $option;
    my $target1 = $1;
    my $target2 = $2;
    $target1 = $1 if($target1 =~ /^"(.*)"$/);
    $target2 = $1 if($target2 =~ /^"(.*)"$/);
    $cost_edge{"$target1 -> $target2"} = $option;
    $g->add_edge($target1,$target2);
    if (defined $predecessors{$target2}){
      push @{$predecessors{$target2}}, $target1;
    }else{
      $predecessors{$target2} = [$target1];
    }
    if (defined $successors{$target1}){
      push @{$successors{$target1}}, $target2;
    }else{
      $successors{$target1} = [$target2];
    }
  }
}
close(FILE);

print "The graph is $g\n";
if( $g->is_dag){
  print "yes it's a DAG\n";
}else{
  print "bad luck  it's not a DAG\n";
}
#if ($g->check_transitive()){
#print "yes\n" 
#}else{
#print "no\n" 
#}
my $m = $g->modular_decomposition_EGMS;
print Graph::ModularDecomposition::tree_to_string( $m );
if(not defined $default_size_node){
	print " the default value for node is now : \n"
}
if(not defined $default_size_edge){
	print " the default value for edge is now : \n"
}

open( FILE, "> $output_file");
print FILE "digraph g {\n";
print FILE "\t node [style=filled]; \n";
print FILE "\t edge [$default_size_edge]; \n";
my $key;
foreach $key (keys %cost_node){
	my $option = $cost_node{$key};
  if(defined $group{$key}){
    $option =  $option.",color=blue" if($group{$key} == 0);
    $option =  $option.",color=red" if($group{$key} == 1);
    $option =  $option.",color=green" if($group{$key} == 2);
  }
	print FILE "\t $key [$option]; \n" if ($option !~ /^1$/);
	print FILE "\t $key; \n" unless ($option !~ /^1$/);
}
foreach $key (keys %cost_edge){
	my $option = $cost_edge{$key};
	print FILE "\t $key [$option]; \n" if ($option !~ /^1$/);
	print FILE "\t $key; \n" unless ($option !~ /^1$/);
}
print FILE "}";
close(FILE);
