#include <unistd.h>
#include <stdio.h>
#include "xbt/sysdep.h"         /* calloc, printf */
#include <time.h>
XBT_LOG_NEW_DEFAULT_CATEGORY(m,
		"Messages specific for this msg example");


/* Create a log channel to have nice outputs. */
#include "msg/msg.h"            /* Yeah! If you want to use msg, you need to include msg/msg.h */


#define NO_TASK ((void*)221267)        /* a magic number to tell thief that there aren't task here */


m_host_t *worker = NULL;
int slaves_count = 0;


int execute_task(int argc, char *argv[]);
int answer_request(int argc, char *argv[]);
MSG_error_t test_all(const char *platform_file, const char *application_file);


int intialisation_data(int argc, char *argv[]){
	slaves_count = argc - 1;
	int i;
	if( worker == NULL ){                             /* Process organisation */
		worker = xbt_new0(m_host_t, slaves_count);

		for (i = 0; i < slaves_count; i++) {
			worker[i] = MSG_get_host_by_name(argv[i+1]);
			xbt_assert2(worker[i] != NULL, "Unknown host %s. i=%d Stopping Now! ",
					argv[i], i);
		}
	}
	for (i = 0; i < slaves_count; i++){
		DEBUG1("%s", worker[i]->name);
		if (MSG_host_self() == worker[i]) {
			return i;
		}
	}
}



/** execute task in the order  */
int execute_task(int argc, char *argv[])
{
	MSG_error_t res;
	int id = intialisation_data( argc,argv);
	m_task_t  task_tmp   = NULL;


	while(1){
		task_tmp = (m_task_t) steal(id);
		INFO1("Pop \"%s\"", MSG_task_get_name(task_tmp));
		task_tmp = NULL;
	}
	return 0;
}                               /* end_of_execute_work */



/** Receiver function  */
int answer_request(int argc, char **argv)
{
	int i;
	m_task_t task;
	m_task_t send_task;
	MSG_error_t res;
	m_host_t host = MSG_host_self();
	const char * hostname = MSG_host_get_name(host);
	INFO1("hostname : %s", hostname);
	char mailbox[64];

	while (1) {
		task=NULL;
		sprintf(mailbox, "%s_task", hostname);
		res = MSG_task_receive(&task, mailbox);
		xbt_assert0(res == MSG_OK, "MSG_task_get failed");
		INFO1("Received \"%s\"", MSG_task_get_name(task));

		m_host_t theif_id = MSG_task_get_source	(task); 
		const char * hostname_theif = MSG_host_get_name(theif_id);
		sprintf(mailbox, "%s_exe", hostname_theif);

		INFO1("Processing \"%s\"", MSG_task_get_name(task));
		MSG_task_execute(task);
		INFO1("\"%s\" done", MSG_task_get_name(task));


		task = NULL;
		send_task = NULL;
		char * name = "NO_Task";
		send_task = MSG_task_create(name, 0, 0, NO_TASK);
		INFO2("Sending \"%s\" to \"%s\"", send_task->name, hostname_theif);
		MSG_task_send_with_timeout(send_task, mailbox, 1);
		INFO0("Sent");
	}
	INFO0("I'm done. See you!");
	return 0;
}                               /* end_of_slave */

/** Test function */
MSG_error_t test_all(const char *platform_file, const char *application_file)
{
	MSG_error_t res = MSG_OK;

	{                             /*  Simulation setting */
		MSG_create_environment(platform_file);
	}
	{                             /*   Application deployment */
		MSG_function_register("execute_task", execute_task);
		MSG_function_register("answer_request", answer_request);
		MSG_launch_application(application_file);
	}
	res = MSG_main();

	INFO1("Simulation time %g", MSG_get_clock());
	free(worker);
	return res;
}                               /* end_of_test_all */


/** Main function */
int main(int argc, char *argv[])
{
	MSG_error_t res = MSG_OK;
	MSG_global_init(&argc, argv);
	if (argc < 3) {
		printf("Usage: %s platform_file deployment_file \n", argv[0]);
		printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
		exit(1);
	}
	res = test_all(argv[1], argv[2]);
	MSG_clean();

	if (res == MSG_OK)
		return 0;
	else
		return 1;
}                               /* end_of_main */
